package serverClient;
import java.io.*;
import java.net.*;
import commands.*;

/**
 * This thread on server-side is supervising the output stream of the server socket and responsable for sending to the client.
 * @author Winkler
 */

public class S_toClient extends Thread
{
	private Socket s;
	private int countOfTicks=620;
	private S_fromClient input;
	public boolean kill = false;

	private int idi;

	private S_common commonThings;

	public S_toClient(Socket cs, int id, S_common commonThings)
	{
		idi = id;
		s=cs;
		this.commonThings = commonThings;
	}

	/**
	 *  This method is responsable for sending the Stone-Objects (and creating them if necessary),
	 *  the new Scores (to the connected Clients) and fall-Commands on every tick.
	 *  This also implies the game start (by sending the first two stones).
	 */

	public void run()
	{
		ObjectInputStream fromClient;
		ObjectOutputStream objectToClient;
		//Variable die jedes Mal um 1 erhoeht wird wenn der Client einen Stein anfordert.
		//int n = 0;

		try
		{
			//ein Inputstream fuer den jeweiligen Client wird geoeffnet
			fromClient = new ObjectInputStream(s.getInputStream());

			//ein neues Objekt wird angelegt, das die vom Client eingehenden Aufgaben verwaltet und verarbeitet.
			input = new S_fromClient(fromClient, this, idi, commonThings);
			//der entsprechende Thread wird gestartet
			input.start();
			//ein Outputstream fuer den jeweiligen Client wird geoeffnet
			objectToClient = new ObjectOutputStream( s.getOutputStream() );

			//Der Thread laeuft solange er nicht "gekillt" wird
			while(!kill)
			{	
				//die Variable commonThing.ticker.i wird von einem TimerTask jede halbe Sekunde von 620 um je 1 herabgezaehlt
				//...das soll synchronisieren...wurde schon herabgezaehlt?
				if (countOfTicks>commonThings.ticker.getI())
				{
					/*
					//...sind wir gerade an dem Punkt wo das Spiel beginnen soll?
					if(commonThings.ticker.i==599)
					{
						objectToClient.reset();
						objectToClient.writeObject(new S_newStonesCommand(commonThings.stones));
						objectToClient.flush();
						System.out.println("newStonesCommand wurde gesandt an Client: " + idii+" with stones size() = "+commonThings.stones.size());
					}
					 */
					//jede halbe Sekunde soll der Stein fallen
					objectToClient.writeObject(new S_fallCommand(countOfTicks));
					objectToClient.flush();


					//damit die Schleife erst wieder beim naechsten "Tick" durchlaufen wird
					countOfTicks=commonThings.ticker.getI();
				}

				//sollte der Client an der Reihe sein, einen neuen Punktestand zu verschicken...
				if(commonThings.newStones == idi)
				{
					sendNewStonesCommand(objectToClient);
				}

				if(commonThings.newScore == idi)
				{
					sendPrintScoreCommand(objectToClient);
				}

				//...damit die while-Schleife nicht ununterbrochen durchrattert und den Rechner ueberlastet
				Thread.sleep(10);
			}
			//geht der Thread zu Ende, wird der Socket geschlossen
			s.close();
			System.out.println(S_common.playerNames[idi-1] + " disconnected!");
		}
		catch( Exception e )
		{
			System.out.println("S_toClient execption!!!");
		}
	}



	public void sendPrintScoreCommand(ObjectOutputStream objectToClient)
	{
		try
		{
			//dann soll er ihn auch verschicken
			objectToClient.reset();
			objectToClient.writeObject(new S_printScoreCommand(S_common.tempPlayerNames, S_common.tempPlayerScores, S_common.tempGameStates));
			objectToClient.flush();
			//System.out.println("newScoreCommand wurde gesandt an Client: " + idi);
			//wenn er noch nicht der letzte Client in der Reihe ist, dann...
			if (idi < commonThings.numberOfClients)
			{
				//...suche den naechsten noch verbundenen Client
				int a = idi;
				while(a < commonThings.numberOfClients)
				{
					if(S_common.gameStates[a].compareTo("Disconnected") == 0)
					{
						a++;
					}
					else
					{
						//...so es noch einen gibt, dann soll sein Serverthread ihm den Punktestand senden
						commonThings.newScore = a + 1;
						//...damit er aus der Schleife hinausfaellt
						a = commonThings.numberOfClients;
					}
				}
			}
			else
			{
				//...sollte es keinen verbundenen Client mehr geben, soll auch keiner senden
				commonThings.newScore = 0;
			}

		}
		catch(Exception e)
		{

		}
	}

	public void sendNewStonesCommand(ObjectOutputStream objectToClient)
	{
		try
		{
			//dann soll er ihn auch verschicken
			objectToClient.reset();
			objectToClient.writeObject(new S_newStonesCommand(commonThings.stones));
			objectToClient.flush();
			//System.out.println("newStonesCommand wurde gesandt an Client: " + idi);
			//wenn er noch nicht der letzte Client in der Reihe ist, dann...
			if (idi < commonThings.numberOfClients)
			{
				//...suche den naechsten noch verbundenen Client
				int a = idi;
				while(a < commonThings.numberOfClients)
				{
					if(S_common.gameStates[a].compareTo("Disconnected") == 0)
					{
						a++;
					}
					else
					{
						//...so es noch einen gibt, dann soll sein Serverthread ihm den Punktestand senden
						commonThings.newStones = a + 1;
						//...damit er aus der Schleife hinausfaellt
						a = commonThings.numberOfClients;
					}
				}
			}
			else
			{
				//...sollte es keinen verbundenen Client mehr geben, soll auch keiner senden
				commonThings.stones = null;
				commonThings.newStones = 0;
			}
		}
		catch(Exception e)
		{

		}
	}
}
